@@ -4,6 +4,7 @@ import six
 import logging
 from collections import Sequence
 from armet import http
+from armet.exceptions import ValidationError
 from armet.resources.resource import base
 
 
@@ -32,11 +33,8 @@ class ManagedResource(base.Resource):
             r'(?:\.(?P<extensions>[^/]+?))??/??$'
         ]
 
-    #! An ordered dictionary of attributes that are gathered from both
-    #! the metaclass and attributes defined directly on the resource.
-    #!
-    #! Attributes may be declared in a few different
-    #! ways. The next few code blocks are identical in function.
+    #! An ordered dictionary of attributes that are gathered from
+    #! attributes defined directly on the resource.
     #!
     #! @code
     #! from armet import resources
@@ -45,38 +43,6 @@ class ManagedResource(base.Resource):
     #!     name = resources.Attribute()
     #!     created = resources.Attribute('created')
     #! @endcode
-    #!
-    #! @code
-    #! from armet import resources
-    #!
-    #! class Resource(resources.Resource):
-    #!     class Meta:
-    #!         include = {
-    #!             'name': resources.Attribute()
-    #!             'created': resources.Attribute('created')
-    #!         }
-    #! @endcode
-    #!
-    #! @code
-    #! from armet import resources
-    #!
-    #! class Resource(resources.Resource):
-    #!     class Meta:
-    #!         include = {
-    #!             'name': None,
-    #!             'created': 'created'
-    #!         }
-    #! @endcode
-    #!
-    #! @code
-    #! from armet import resources
-    #!
-    #! class Resource(resources.Resource):
-    #!     class Meta:
-    #!         include = ('name',)
-    #!
-    #!     created = resources.Attribute('created')
-    #! @endcode
     attributes = None
 
     #! The slug that is used to identify the resource an item is being
@@ -104,6 +70,12 @@ class ManagedResource(base.Resource):
         # Return what we got.
         return result
 
+    def __init__(self, *args, **kwargs):
+        super(ManagedResource, self).__init__(*args, **kwargs)
+
+        #! Map of errors that have occurred in the clean cycle.
+        self._errors = {}
+
     @property
     def allowed_operations(self):
         """Retrieves the allowed operations for this request."""
@@ -134,8 +106,13 @@ class ManagedResource(base.Resource):
             # No data; return nothing.
             return None
 
-        if (not isinstance(data, six.string_types)
-                and isinstance(data, Sequence)):
+        if self.path:
+            # Ensure we are dealing with only one segment.
+            if '/' in self.path:
+                raise http.exceptions.NotFound()
+
+        if (isinstance(data, Sequence)
+                and not isinstance(data, six.string_types)):
             # Attempt to prepare each item of the iterable (as long as
             # we're not a string or some sort of mapping).
             for index, value in enumerate(data):
@@ -144,9 +121,32 @@ class ManagedResource(base.Resource):
             return data
 
         # Prepare just the singular value and return.
-        return self.item_prepare(data)
+        data = self.item_prepare(data)
+        if data is None:
+            raise http.exceptions.NotFound()
+
+        return data
+
+    def attribute_prepare(self, name, attribute, item):
+        # Run the attribute through its prepare cycle.
+        return attribute.prepare(
+            # Optional preparation cycle on the resource.
+            self.preparers[name](
+                # Retrieves the value from the object.
+                self, item, attribute.get(item)))
 
     def item_prepare(self, item):
+        # Check for a path first.
+        if self.path:
+            attribute = self.attributes.get(self.path)
+            if attribute is None:
+                raise http.exceptions.NotFound()
+
+            if not attribute.read:
+                raise http.exceptions.Forbidden()
+
+            return self.attribute_prepare(self.path, attribute, item)
+
         # Initialize the object that hold the resultant item.
         obj = {}
 
@@ -154,22 +154,38 @@ class ManagedResource(base.Resource):
         for name, attribute in self.attributes.items():
             if attribute.include:
                 # Run the attribute through its prepare cycle.
-                obj[name] = attribute.prepare(
-                    # Optional preparation cycle on the resource.
-                    self.preparers[name](
-                        # Retrieves the value from the object.
-                        self, item, attribute.get(item)))
+                obj[attribute.name] = self.attribute_prepare(
+                    name, attribute, item)
 
         # Return the resultant object.
         return obj
 
+    def _clean(self, data):
+        # Wrap clean so that it can be extended and have validation
+        # errors properly handled.
+
+        try:
+            data = self.clean(data)
+
+        except AssertionError as ex:
+            self._errors['__all__'] = [str(ex)]
+
+        except ValidationError as ex:
+            self._errors['__all__'] = ex.errors
+
+        if self._errors:
+            # Collect errors and raise a BadRequest message to the client.
+            raise http.exceptions.BadRequest(self._errors)
+
+        return data
+
     def clean(self, data):
         if not data:
-            # If no data; return an empty object.
-            return {}
+            # If no data; just alias it to an empty dictionary.
+            data = {}
 
-        if (not isinstance(data, six.string_types)
-                and isinstance(data, Sequence)):
+        if (isinstance(data, Sequence)
+                and not isinstance(data, six.string_types)):
             # Attempt to clean each item of the iterable (as long as
             # we're not a string or some sort of mapping).
             for index, value in enumerate(data):
@@ -181,18 +197,41 @@ class ManagedResource(base.Resource):
         return self.item_clean(data)
 
     def item_clean(self, item):
-        # Initialize the object that hold the resultant item.
+        # Iterate through the attributes and build the object from the item.
         obj = {}
 
-        # Iterate through the attributes and build the object from the item.
         for name, attribute in self.attributes.items():
-            if attribute.include:
-                # Run the attribute through its prepare cycle.
-                obj[name] = self.cleaners[name](
-                    # Micro preparation cycle on the attribute object.
-                    self, attribute.clean(item.get(name)))
+            value = item.get(attribute.name)
+
+            try:
+                if value is not None:
+                    # Check if this attribute is writeable.
+                    if not attribute.write:
+                        raise ValidationError('Attribute is read-only.')
+
+                    # Run the attribute through its clean cycle.
+                    value = self.cleaners[name](
+                        # Micro preparation cycle on the attribute object.
+                        self, attribute.clean(value))
+
+                # Ensure that we don't have a null or it is provided.
+                if value is None:
+                    if name in item and not attribute.null:
+                        raise ValidationError('Must not be null.')
+
+                    if name not in item and attribute.required:
+                        raise ValidationError('Must be provided.')
+
+            except AssertionError as ex:
+                self._errors[attribute.name] = [str(ex)]
+                value = None
+
+            except ValidationError as ex:
+                self._errors[attribute.name] = ex.errors
+                value = None
+
+            obj[name] = value
 
-        # Return the resultant object.
         return obj
 
     @property
@@ -220,17 +259,19 @@ class ManagedResource(base.Resource):
         # Ensure we're allowed to read the resource.
         self.assert_operations('read')
 
-        try:
-            # Delegate to `read` to retrieve the items.
-            items = self.read()
-
-        except AttributeError:
-            # No read method defined.
-            raise http.exceptions.NotImplemented()
+        # Delegate to `read` to retrieve the items.
+        items = self.read()
 
         if self.slug is not None and not items:
             # Requested a specific resource but nothing is returned.
-            raise http.exceptions.NotFound()
+
+            # Attempt to resolve by changing what we understand as
+            # a slug to a path.
+            self.path = self.path + self.slug if self.path else self.slug
+            self.slug = None
+
+            # Attempt to retreive the resource again.
+            items = self.read()
 
         # Build the response object.
         self.make_response(items)
@@ -245,15 +286,10 @@ class ManagedResource(base.Resource):
         self.assert_operations('create')
 
         # Deserialize and clean the incoming object.
-        data = self.clean(self.request.read(deserialize=True))
+        data = self._clean(self.request.read(deserialize=True))
 
-        try:
-            # Delegate to `create` to create the item.
-            item = self.create(data)
-
-        except AttributeError:
-            # No read method defined.
-            raise http.exceptions.NotImplemented()
+        # Delegate to `create` to create the item.
+        item = self.create(data)
 
         # Build the response object.
         self.make_response(item, status=http.client.CREATED)
@@ -264,16 +300,11 @@ class ManagedResource(base.Resource):
             # Mass-PUT is not implemented.
             raise http.exceptions.NotImplemented()
 
-        try:
-            # Check if the resource exists.
-            target = self.read()
-
-        except AttributeError:
-            # No read method defined.
-            raise http.exceptions.NotImplemented()
+        # Check if the resource exists.
+        target = self.read()
 
         # Deserialize and clean the incoming object.
-        data = self.clean(self.request.read(deserialize=True))
+        data = self._clean(self.request.read(deserialize=True))
 
         if target is not None:
             # Ensure we're allowed to update the resource.
@@ -294,13 +325,8 @@ class ManagedResource(base.Resource):
             # Ensure we're allowed to create the resource.
             self.assert_operations('create')
 
-            try:
-                # Delegate to `create` to create the item.
-                target = self.create(data)
-
-            except AttributeError:
-                # No read method defined.
-                raise http.exceptions.NotImplemented()
+            # Delegate to `create` to create the item.
+            target = self.create(data)
 
             # Build the response object.
             self.make_response(target, status=http.client.CREATED)
@@ -314,13 +340,8 @@ class ManagedResource(base.Resource):
         # Ensure we're allowed to destroy a resource.
         self.assert_operations('destroy')
 
-        try:
-            # Delegate to `destroy` to destroy the item.
-            self.destroy()
-
-        except AttributeError:
-            # No read method defined.
-            raise http.exceptions.NotImplemented()
+        # Delegate to `destroy` to destroy the item.
+        self.destroy()
 
         # Build the response object.
         self.make_response(status=http.client.NO_CONTENT)
